"use strict";

var _fs = require("fs");

var _common = require("@srejs/common");

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var loaderDefaultArr = ['.vue', '.js', '.jsx', '.ts', '.tsx', '.css', '.scss', '.less', '.png', '.jpg', '.jpeg', '.gif', '.svg'];

function isSameLoader(loader1, loader2) {
  var loader1Name = typeof loader1 === 'string' ? loader1 : loader1.loader;
  var loader2Name = typeof loader2 === 'string' ? loader2 : loader2.loader;
  return loader1Name === loader2Name;
}

function mergeLoader(before, item) {
  if (item && item.length) {
    //如果有则覆盖，没有则添加 双指针：一个beforeIndex指向原有默认配置，一个itemIndex指向用户配置
    //1.loader相同 beforeIndex++ && itemIndex++,覆盖
    //2.loader不相同 beforeIndex++,添加
    var beforeLength = before.use.length,
        itemLength = item.length;
    var beforeIndex = 0,
        itemIndex = 0;

    while (beforeIndex < beforeLength && itemIndex < itemLength) {
      if (isSameLoader(before.use[beforeIndex], item[itemIndex])) {
        before.use[beforeIndex] = item[itemIndex];
        beforeIndex++;
        itemIndex++;
      } else {
        beforeIndex++;
      }
    }

    if (itemIndex < itemLength) {
      var _before$use;

      //用户配置
      (_before$use = before.use).push.apply(_before$use, _toConsumableArray(item.slice(itemIndex)));
    }
  }
} //loader单独处理


function loaderConfig(userConfigLoader, config) {
  //添加loader
  var defaultLoader = config.module.rules;
  userConfigLoader && Object.entries(userConfigLoader).map(function (_ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        key = _ref2[0],
        item = _ref2[1];

    switch (key) {
      case 'vue':
        mergeLoader(defaultLoader[0], item);
        break;

      case 'js':
        mergeLoader(defaultLoader[1], item);
        break;

      case 'jsx':
        mergeLoader(defaultLoader[2], item);
        break;

      case 'ts':
        mergeLoader(defaultLoader[3], item);
        break;

      case 'tsx':
        mergeLoader(defaultLoader[4], item);
        break;

      case 'css':
        mergeLoader(defaultLoader[5], item);
        break;

      case 'scss':
        mergeLoader(defaultLoader[6], item);
        break;

      case 'less':
        mergeLoader(defaultLoader[7], item);
        break;

      case 'img':
        mergeLoader(defaultLoader[8], item);
        break;

      case 'other':
        //没有默认loader的文件添加
        item.length && item.map(function (loader) {
          !loaderDefaultArr.some(function (loaderDefault) {
            return loaderDefault.match(loader.test);
          }) && defaultLoader.push(loader);
        });
        break;

      default:
    }
  });
}

module.exports = function (config) {
  if (!_fs.existsSync(_common.webpackConfigPath)) {
    return config;
  }

  delete require.cache[require.resolve(_common.webpackConfigPath)];

  var customConfig = require(_common.webpackConfigPath);

  if (!customConfig) return config;
  Object.entries(customConfig).map(function (_ref3) {
    var _ref4 = _slicedToArray(_ref3, 2),
        key = _ref4[0],
        val = _ref4[1];

    switch (key) {
      case 'loader':
        loaderConfig(val, config);
        break;

      case 'externals':
        //添加externals 合并
        config.externals = val ? Object.assign(config.externals || {}, val) : config.externals;
        break;

      case 'extensions':
        //添加extensions 合并去重
        config.resolve.extensions = val && val.length ? Array.from(new Set([].concat(_toConsumableArray(config.resolve.extensions), _toConsumableArray(val)))) : config.resolve.extensions;
        break;

      case 'alias':
        //添加alias 覆盖
        config.resolve.alias = val || config.resolve.alias;
        break;

      case 'plugins':
        //添加plugin 合并
        config.plugins = val ? config.plugins.concat(val) : config.plugins;
        break;

      default:
        //直接覆盖操作
        config[key] = val || config[key] || '';
        break;
    }
  });
  return config;
};